/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file Singleton.C
 *    Implementation of SingletonSharedLibraryHelper.
 *
 * @author Erik Einhorn
 * @date   2011/02/01
 */

#include <utils/Singleton.h>

#include <list>
#include <map>
#include <string>

namespace mira { namespace Private {

///////////////////////////////////////////////////////////////////////////////

#ifdef MIRA_WINDOWS

struct SingletonRegistry
{
	void* getSingleton(const std::type_info& type)
	{
		std::string typeStr = type.name();
		auto it = singletons.find(typeStr);
		if(it==singletons.end())
			return NULL;
		else
			return it->second;
	}

	void setSingleton(const std::type_info& type, void* singleton)
	{
		std::string typeStr = type.name();
		singletons[typeStr] = singleton;
	}

	std::map<std::string,void*> singletons;

	// here we are safe to use a single static variable even with those bloody
	// Windows DLLs
	static SingletonRegistry& instance()
	{
		static SingletonRegistry sRegistry;
		return sRegistry;
	}
};

void* SingletonSharedLibraryHelper::getSharedLibrarySingleton(const std::type_info& type)
{
	return SingletonRegistry::instance().getSingleton(type);
}

void SingletonSharedLibraryHelper::setSharedLibrarySingleton(const std::type_info& type,
                                                             void* singleton)
{
	SingletonRegistry::instance().setSingleton(type, singleton);
}

#endif

///////////////////////////////////////////////////////////////////////////////

}}
