/*
 * Copyright (C) by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file ChannelSynchronizerTest.C
 *    Test ChannelSynchronizer
 *
 * @author Christof Schröter
 * @date   2020/06/24
 */

#include <boost/test/unit_test.hpp>

#include <fw/Framework.h>

using namespace mira;

BOOST_AUTO_TEST_CASE(ChannelSynchronizerTest)
{
	const char* argv[] = {"ChannelSynchronizerTest",
	                      "${find ChannelSynchronizerTest.xml}"};
	Framework fw(2,(char**)argv);
	fw.load();
	fw.start();

	Time start = Time::now();

	Authority auth;
	auth.checkin("/TestAuthority");

	Channel<int> tsSynchCount = auth.subscribe<int>("/ByTimestamp/SynchronizedCount");
	BOOST_REQUIRE(auth.waitForData(tsSynchCount, Duration::seconds(5)).isValid());

	Channel<int> seqSynchCount = auth.subscribe<int>("/BySequenceID/SynchronizedCount");
	BOOST_REQUIRE(auth.waitForData(seqSynchCount, Duration::seconds(5)).isValid());

	while (Time::now() - start < Duration::seconds(10)) {

		int tsCount = tsSynchCount.get();
		int seqCount = seqSynchCount.get();
		std::cout << tsCount << " " << seqCount << std::endl;

		if ((tsCount > 2) && (seqCount > 4))
			return; // success

		MIRA_SLEEP(1000);
	}

	BOOST_FAIL("Not enough syncs");
}
